<?php

/**
 * Define rules compatible actions.
 *
 * This hook is required in order to add a new rules action. It should be
 * placed into the file MODULENAME.rules.inc, which gets automatically included
 * when the hook is invoked.
 *
 * @return
 *   An array of information about the module's provided rules actions.
 *   The array contains a sub-array for each action, with the action name as
 *   the key. Actions names may only contain lowercase alpha-numeric characters
 *   and underscores and should be prefixed with the providing module name.
 *   Possible attributes for each sub-array are:
 *   - label: The label of the action. Start capitalized. Required.
 *   - group: A group for this element, used for grouping the actions in the
 *     interface. Should start with a capital letter and be translated.
 *     Required.
 *   - parameter: (optional) An array describing all parameter of the action
 *     with the parameter's name as key. Each parameter has to be
 *     described by a sub-array with possible attributes as described
 *     afterwards, whereas the name of a parameter needs to be a lowercase,
 *     valid PHP variable name.
 *   - provides: (optional) An array describing the variables the action
 *     provides to the evaluation state with the variable name as key. Each
 *     variable has to be described by a sub-array with possible attributes as
 *     described afterwards, whereas the name of a parameter needs to be a
 *     lowercase, valid PHP variable name.
 *   - 'named parameter': (optional) If set to TRUE, the arguments will be
 *     passed as a single array with the parameter names as keys. This emulates
 *     named parameters in PHP and is in particular useful if the number of
 *     parameters can vary. Defaults to FALSE.
 *   - base: (optional) The base for action implementation callbacks to use
 *     instead of the action's name. Defaults to the action name.
 *   - callbacks: (optional) An array which allows to set specific function
 *     callbacks for the action. The default for each callback is the actions
 *     base appended by '_' and the callback name.
 *   - 'access callback': (optional) A callback which has to return whether the
 *     currently logged in user is allowed to configure this action. See
 *     rules_node_integration_access() for an example callback.
 *  Each 'parameter' array may contain the following properties:
 *   - label: The label of the parameter. Start capitalized. Required.
 *   - type: The rules data type of the parameter, which is to be passed to the
 *     action. All types declared in hook_rules_data_info() may be specified, as
 *     well as an array of possible types. Also lists and lists of a given type
 *     can be specified by using the notating list<integer> as introduced by
 *     the entity metadata module, see hook_entity_property_info(). The special
 *     keyword '*' can be used when all types should be allowed. Required.
 *   - bundles: (optional) An array of bundle names. When the specified type is
 *     set to a single entity type, this may be used to restrict the allowed
 *     bundles.
 *   - description: (optional) If necessary, a further description of the
 *     parameter.
 *   - options list: (optional) A callback that returns an array of possible
 *     values for this parameter. The callback has to return an array as used
 *     by hook_options_list(). For an example implementation see
 *     rules_data_action_type_options().
 *   - save: (optional) If this is set to TRUE, the parameter will be saved by
 *     rules when the rules evaluation ends. This is only supported for savable
 *     data types. If the action returns FALSE, saving is skipped.
 *   - optional: (optional) May be set to TRUE, when the parameter isn't
 *     required.
 *   - 'default value': (optional) The value to pass to the action, in case the
 *     parameter is optional and there is no specified value.
 *   - 'allow null': (optional) Usually Rules will not pass any NULL values as
 *     argument, but abort the evaluation if a NULL value is present. If set to
 *     TRUE, Rules will not abort and pass the NULL value through. Defaults to
 *     FALSE.
 *   - restriction: (optional) Restrict how the argument for this parameter may
 *     be provided. Supported values are 'selector' and 'input'.
 *   - default mode: (optional) Customize the default mode for providing the
 *     argument value for a parameter. Supported values are 'selector' and
 *     'input'. The default depends on the required data type.
 *   - sanitize: (optional) Allows parameters of type 'text' to demand an
 *     already sanitized argument. If enabled, any user specified value won't be
 *     sanitized itself, but replacements applied by input evaluators are as
 *     well as values retrieved from selected data sources.
 *   - translatable: (optional) If set to TRUE, the provided argument value
 *     of the parameter is translatable via i18n String translation. This is
 *     applicable for textual parameters only, i.e. parameters of type 'text',
 *     'token', 'list<text>' and 'list<token>'. Defaults to FALSE.
 *   - ui class: (optional) Allows overriding the UI class, which is used to
 *     generate the configuration UI of a parameter. Defaults to the UI class of
 *     the specified data type.
 *   - cleaning callback: (optional) A callback that input evaluators may use
 *     to clean inserted replacements; e.g. this is used by the token evaluator.
 *   - wrapped: (optional) Set this to TRUE in case the data should be passed
 *     wrapped. This only applies to wrapped data types, e.g. entities.
 *  Each 'provides' array may contain the following properties:
 *   - label: The label of the variable. Start capitalized. Required.
 *   - type: The rules data type of the variable. All types declared in
 *     hook_rules_data_info() may be specified. Types may be parametrized e.g.
 *     the types node<page> or list<integer> are valid.
 *   - save: (optional) If this is set to TRUE, the provided variable is saved
 *     by rules when the rules evaluation ends. Only possible for savable data
 *     types. Defaults to FALSE.
 *
 *  The module has to provide an implementation for each action, being a
 *  function named as specified in the 'base' key or for the execution callback.
 *  All other possible callbacks are optional.
 *  Supported action callbacks by rules are defined and documented in the
 *  RulesPluginImplInterface. However any module may extend the action plugin
 *  based upon a defined interface using hook_rules_plugin_info(). All methods
 *  defined in those interfaces can be overridden by the action implementation.
 *  The callback implementations for those interfaces may reside in any file
 *  specified in hook_rules_file_info().
 *
 *  @see hook_rules_file_info()
 *  @see rules_action_execution_callback()
 *  @see hook_rules_plugin_info()
 *  @see RulesPluginImplInterface
 */
function redmine_sync_rules_action_info() {
  $return = array(
    'create_time_entry' => array(
      'label' => t('Create Time Entry'),
      'parameter' => array(
        'mail' => array(
          'type'     => 'text', 
          'label'    => t('Redmine user email'),
        ),
        'issue_id' => array(
          'type'     => 'integer', 
          'label'    => t('Redmine Issue ID'),
        ),
        'spent_on' => array(
          'type'     => 'date', 
          'label'    => t('Time Entry date'),
        ),
        'hours' => array(
          'type'     => 'decimal', 
          'label'    => t('Hours'),
        ),
        'comment' => array(
          'type'     => 'text', 
          'label'    => t('Time Entry comment'),
          'optional' => TRUE,
        ),
        'activity' => array(
          'type'         => 'integer',
          'label'        => t('Activity'),
          'options list' => '_redmine_sync_rules_get_activities',
        ),
      ),
      'group' => t('Redmine Sync'),
      'callbacks' => array(
        'help'     => 'redmine_sync_rules_time_entry_help',
        'validate' => 'redmine_sync_rules_time_entry_validate',
        'execute'  => 'redmine_sync_rules_time_entry_create',
      ),
    ),
  );

  // Adding the custom fields.
  $rdm_custom_fields = redmine_sync_rest_get_custom_fields('time_entry', REDMINE_SYNC_RET_VAL_TYPE_DEFAULT);
  foreach ($rdm_custom_fields as $c_field) {
    $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']] = array(
      'label'     => $c_field['name'],
      'optional'  => $c_field['is_required'] == 'true' ? false : true,
      'rdm_id'    => $c_field['id'],
      'is_custom' => true,
    );
    switch ($c_field['field_format']) {
      case 'bool':
        $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']]['type'] = 'boolean';
        $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']]['default value'] = $c_field['default_value'] instanceof SimpleXMLElement ? null : (bool)(int)$c_field['default_value'];
        break;
      case 'date':
        $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']]['type'] = 'date';
        $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']]['default value'] = (string)$c_field['default_value'];
        break;
      case 'float':
        $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']]['type'] = 'decimal';
        $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']]['default value'] = (float)$c_field['default_value'];
        break;
      case 'int':
        $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']]['type'] = 'integer';
        $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']]['default value'] = (int)$c_field['default_value'];
        break;
      case 'list': // @todo: Required more tests!!!
        $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']]['type'] = 'list';
        $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']]['default value'] = (string)$c_field['default_value'];
        break;
      case 'link':
        $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']]['type'] = 'uri';
        $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']]['default value'] = (string)$c_field['default_value'];
        break;
      case 'user':
      case 'version':
        $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']]['type'] = 'integer';
        $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']]['label'].= ' (ID)';
        break;
      default: // Case for types: text (Redmine "long text"), string (Redmine "text").
        $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']]['type'] = 'text';
        $return['create_time_entry']['parameter']['rdm_custom_' . $c_field['id']]['default value'] = (string)$c_field['default_value'];
    }
  }

  return $return;
}

function _redmine_sync_rules_get_activities(RulesAction $action, $el_name) {
  return redmine_sync_rest_get_time_entry_activities();
}

/**
 * Custom help callback.
 */
function redmine_sync_rules_time_entry_help(RulesAction $action) {
}

/**
 * Custom validation callback for the data set action.
 */
function redmine_sync_rules_time_entry_validate(RulesAbstractPlugin $plugin) {
  // if (empty($plugin->settings['hours'])) {
  //   throw new RulesIntegrityException(t('Error message.'), array($plugin, 'parameter', 'hours'));
  // }
}

/**
 * Custom callback for execute action.
 */
function redmine_sync_rules_time_entry_create() {
  $args = func_get_args();

  // Prepare inputed RulesAction and RulesState.
  $rules_act = null;
  $rules_ste = null;
  foreach ($args as $c_arg) {
    if ($rules_act == null && $c_arg instanceof RulesAction) $rules_act = $c_arg;
    if ($rules_ste == null && $c_arg instanceof RulesState)  $rules_ste = $c_arg;
  }

  // Prepare fields.
  $fields = array();
  if ($rules_act) {
    $rules_act_info = $rules_act->info();
    $fields = $rules_act_info['parameter'];
    foreach ($fields as $c_name => $c_field) {
      $fields[$c_name]['value'] = array_shift($args);
    }
  }

  // Note: if this action started on cron run then redmine_sync_init_rest() return REDMINE_SYNC_REST_API_INIT_NO_USER_KEY
  if (count($fields) && redmine_sync_init_rest() == REDMINE_SYNC_REST_API_INIT_OK) {
    $rdm_user_info = redmine_sync_rest_get_user_by_mail($fields['mail']['value']);
    if ($rdm_user_info) {

      // Create Time entry post array.
      // @todo: check if the user can access to project with selected issue.
      $time_entry_post = array(
        'time_entry' => array(
          'issue_id'    => $fields['issue_id']['value'],
          'spent_on'    => date('Y-m-d', $fields['spent_on']['value']),
          'hours'       => $fields['hours']['value'],
          'comments'    => $fields['comment']['value'],
          'activity_id' => $fields['activity']['value'],
        )
      );

      // Add custom fields values.
      foreach ($fields as $c_field) {
        if (isset($c_field['is_custom']) &&
                  $c_field['is_custom']) {
          $c_buf = array();
          $c_buf['key'] = 'custom_field';
          $c_buf['attributes']['id'] = $c_field['rdm_id'];
          if (is_array($c_field['value'])) {
            $c_buf['attributes']['multiple'] = 'true';
            $c_buf['value'][0] = array('key' => 'value', 'attributes' => array('type' => 'array'));
            foreach ($c_field['value'] as $c_val) {
              $c_buf['value'][0]['value'][] = array('key' => 'value', 'value' => $c_val);
            }
          } else {
            $c_buf['value'] = array('value' => $c_field['value']);
          }            
          $custom_fields_data[]= $c_buf;
        }
      }
      if (count($custom_fields_data)) {
        $time_entry_post['time_entry'][] = array(
          'key'        => 'custom_fields',
          'attributes' => array('type' => 'array'),
          'value'      => $custom_fields_data,
        );
      }

      // Post request.
      $request = redmine_sync_rest_request('time_entries', 'POST', $time_entry_post, $rdm_user_info['login']);

      // Show result message.
      if ($request->code == 201) {
        drupal_set_message(t('New entry %name with id = %id was created.', array('%name' => 'time_entries', '%id' => (int)$request->data_prepared->id)));
        watchdog('redmine_sync', 'New entry %name with id = %id was created.', array('%name' => 'time_entries', '%id' => (int)$request->data_prepared->id));
      } else {
        drupal_set_message(t('Can\'t create new entry %name! Error: %error', array('%name' => 'time_entries', '%error' => $request->error . (isset($request->data_prepared->error) ? ' - '.$request->data_prepared->error : ''))), 'error');
        watchdog('redmine_sync', 'Can\'t create new entry %name! Error: %error', array('%name' => 'time_entries', '%error' => $request->error . (isset($request->data_prepared->error) ? ' - '.$request->data_prepared->error : '')), WATCHDOG_ERROR);
      }
    }
  }
}

